# Tree State Management
每一个state change都有点类似是一个reducer，这整个folder tree的state management有点像一个小型的redux，顶层的folder tree 去管理整个state，每一个action类似于一个reducer： action(oldState, params) => newState.

有点类似于手写了一个redux，然后也碰到了一些redux的坑，让我对redux的理解更深入了。比如说理解了为什么reducer必须是一个pure function，我在写的过程中就遇到了这个bug，更新state in place，结果update了以后发现树没有更新。后来发现在顶层setTreeState的rootnode没有变化，尽管已经有in place的变化，所以没有re-render。

一开始的解决方案是每一个state update都先deep clone一次，确保是100%的pure function。后来继续优化发现，这个不是一个非常严格的reducer，所以不需要很pure，更好的优化就是in place update，然后{ ...rootNode },这样优化了每次要deep clone的时间，从O(n) => O(1)

# 这样设计的好处
我特别喜欢pure function的概念，因为这样可以把每一个action抽象成一个reducer，这样又两个主要的好处：
1）逻辑清晰，非常容易理解，代码也很清晰，每一个reducer对应一个action，想加一个action就加一个reducer
2）容易unit test，这样可以保证运行上不出bug。
  - 之前的老版本设计上就有点混乱，有些action不容易unit test，因为跟整个folder tree的耦合度太高了，所以特别难debug
  - 在写unit test的同时，又是也会直接想到之前写的代码的错误，这个错误可能是手动测试不容易发现的。比如在写open的时候，一开始给每个node都加了open，在测试的时候没什么问题，但是在unit test的时候，把整个state写出来了，就发现，哦原来只有folder才应该有这个属性。所以在测试的时候就发现bug，不用等到人肉测试或者测试component的时候


# 从头到尾build一个工程化的项目
包括eslint，babel，jest，webpack。从中不仅可以学到工程化的知识，还能学到很多细节。比如很多eslint rule有错误，我就会去看为什么不建议这么做，会学到很多细节的知识，同时也会思考，自己做取舍，disable一些自己觉得不需要的eslint rules。

比如for...of的loop，eslint一般不建议这么做，因为这用到了generator。我就用一些工具把for of和.each() 编译成es5，就会看到确实for of会用到generator，所以代码复杂很多。但是经过一些思考，我觉得抛弃这个rule，因为我觉得“让代码快”，和“让代码简洁”是需要作出一些取舍的，比如C++和python，必然是python慢，但是python好写。而且当代的计算机，已近快到可以忽略这么点的“慢”，只要不是算法的问题，比如O(n) => O(n^2)，就不会有大问题。从10ms => 20ms用户根本不会感觉到。

从头到尾完成一个项目，还有一个学到的地方就是各种工程化的工具，流程。webpack是一个十分强大的工具，以前根本不会去注意webpack的细节，都是用create-react-app之类的，现在一点点build webpack的config，学到很多东西，不仅是config，还有怎么优化代码之类的，怎么样减少打包的size。其实可能一半的时间都用在工程化的建设上，以后就会有经验了。


# 内行的好处
有一句话叫做，外行看热闹，内行看门道。我自己写了一个public library以后，看到其它的public library就会感觉到，有好多东西可以学习。之前看别的public library的时候，觉得就是一团乱麻，根本不知道从何下手。现在有了很多的基础知识，基本扫一眼就知道了大概的结构，比如看到别人的webpack config，以前根本一点都看不懂，现在可以好好的读一读，看看有什么地方可以学习的。

有点像是锤子效应：拿着锤子，看什么都像钉子。我现在就是，看到什么东西，都想加到我的code里面。比如看到别人的webpack config，就会想这里有什么好的，我的webpack有什么可以改进的。包括我自己的网站，现在看所有网站，看到有趣的设计或者很炫酷的动画，就会想，我的网站有没有可以借鉴的地方？之前看到那个很酷的彩带动画，我还发邮件给他们网站的联系人，他们网站的联系人还回我了，把代码发给我，这是非常有趣的经历。


# 开发者也是项目经理
还有一点学到的很多东西就是，有时候在工作中，我们会有明确的要求，需要做什么功能和api。但是自己写一个library的好处是，自己既是开发者，又是产品经理。我需要自己去想很多东西，包括怎么样才能符合开发者的需求，同时不让这个library太复杂。更细节的方面，我要想，怎么去方便开发者上手，所以export了testData，因为我在自己使用public library的过程中，有时候看到文档不全面，或者没有例子，就会容易弃用。那我自己就做了一个demo网站，然后如果开发者想直接测试的话，可以直接拿到我的testData，非常方便。



# 以后继续更新的地方
- 像create react app一样，可以建一层canvas，把compile error放到view里面。这样的好处：
  - 不需要去转去console看
  - 作为前端，很重要的一点就是，什么都要好看。在terminal里面的error太难看了，在view里面，就算是报错的信息，也可以处理的很好看。

- 继续看看有什么能够customize，比如以前的版本可以有custome file component和folder component。
